library(diversitree)

correct_ultrametricized_trees <- function(tree){
  depths_of_nodes <- node.depth.edgelength(tree)
  number_of_tips <- length(tree$tip.label)
  depths_of_tips <- depths_of_nodes[1:number_of_tips]
  max_depth <- max(depths_of_tips)
  edges <- tree$edge
  for (i in 1:length(tree$edge.length)){
    if (tree$edge[i, 2] <= number_of_tips){
      tip <- tree$edge[i, 2]
      tree$edge.length[i] <- tree$edge.length[i] + (max_depth - depths_of_tips[tip])
    }
  }
  return (tree)
}

##################################################################################################
get_tree_name <- function(method, phyml, sum_of_logs){
  name <- "dated_treepl.newick"
  if (method == "NPRS"){
    name <- "dated_tree_NPRS.newick"
  }else if (method == "PATHd8"){
    name <- "dated_tree_pathd8.newick"
  }else if (method == "r8s"){
    name = "r8s_dated_tree.newick"
  }else if (method == 'mrbayes'){
    if (sum_of_logs){
      name <- "MAP_tree.tre"
      
    }else{
      name <- "parsemb_map_tree.tre"
      
    }

  }
  if(phyml == 1){
    name <- paste(c("phyML_", name), collapse = "")
  }
  return (name)
}
##################################################################################################
fit_bisse_models <- function(output_dir, phyml, states_path, method, sum_of_logs){
  tree_file_path_name <- get_tree_name(method, phyml, sum_of_logs)
  false_positive <- 0
  file_extention <- ""
  tree_path <- NULL
  tree <- NULL
  states <- readRDS(states_path)
  dir_lrt <- paste(c(output_dir, get_name_of_lrt_dir(method, sum_of_logs)), collapse = "/") # "LRT_test"
  if(!dir.exists(dir_lrt)){
    dir.create(dir_lrt)
  }

  
  if (phyml == 1){
    file_extention <- "phyML_"
  }
  file_extention <- paste(c(file_extention, method, '_'), collapse = "")


  tree_path <-  paste(c(output_dir, tree_file_path_name), collapse = "/")
  print("tree!!")
  print(tree_path)
  tree <- read.tree(tree_path)

  tree <- correct_ultrametricized_trees(tree)
  print(paste(c(tree_path, " ### tree path"), collapse = ""))
  p = starting.point.bisse(tree)
  likelihood_free = make.bisse(tree, states)
  fit_likelihood_free = find.mle(likelihood_free, p)
  likelihood_constraint = constrain(likelihood_free, lambda0 ~ lambda1, mu0 ~ mu1)  # null model
  fit_liklihood_constraint = find.mle(likelihood_constraint, p[argnames(likelihood_constraint)]) # mle of null model
  lrt_results = anova(fit_likelihood_free, fit_liklihood_constraint)
  p_val <- lrt_results[[5]][2] #lrt_results$`Pr(>|Chi|)`[2]
  #AIC_results = AIC(fit_liklihood_constraint, fit_likelihood_free) # calculate AIC
  if (p_val < 0.05){ # if p_value is lower than 0.05 (alpha)-> false positive
    false_positive <- 1
  }
  alternative_parameters <- fit_likelihood_free$par

  estimates <- alternative_parameters
  
  estimates_short_name <- paste(c(file_extention, "estimates.csv"), collapse = "")
  binary_res_file <- paste(c(file_extention, "false_positives.rds"), collapse = "")
  p_value <- paste(c(file_extention, "p_value.rds"), collapse = "")
  
  #mse_res_file <- paste(c(file_extention, "mse_res.csv"), collapse = "")
  false_pos_full_path <- paste(c(dir_lrt, binary_res_file), collapse = "/")
  estimates_full_path <- paste(c(dir_lrt, estimates_short_name), collapse = "/")
  p_value_full_path  <- paste(c(dir_lrt, p_value), collapse = "/")
  #mse_file_full_path <- paste(c(output_dir, mse_res_file), collapse = "/")
  remove_existing_files(c(false_pos_full_path, estimates_full_path))#, mse_file_full_path))
  saveRDS(false_positive, false_pos_full_path)
  saveRDS(p_val, p_value_full_path)
  write.table(estimates, file = estimates_full_path ,col.names = FALSE, sep = ",")

}

###########################################################################################################

get_name_of_lrt_dir <- function(method, sum_of_logs){
  if (method == "mrbayes"){
    if (sum_of_logs){
      return ('LRT_test_sum_ln')
    }else{
      return ('LRT_test_ln')
    }
    
  }
  return ('LRT_test')
}
############################################################################################################
remove_existing_files <- function(paths){
  for (i in 1:length(paths)){
    if (file.exists(paths[i])){
      file.remove(paths[i])
    }
  }
}

############################################################################################################
sum_of_logs <- 0
args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
}
fit_bisse_models(output_dir, phyml, states_path, method, sum_of_logs)

