library(phangorn)

calculate_tree_distance <- function(simulation_dir, res_dir, list_of_r, list_of_taxa, number_of_replicates, method, sub_dir, inner_dir, phyML){

  number_of_taxa_cat <- length(list_of_taxa)
  number_of_r_values <- length(list_of_r)

  matrix_res <- matrix(nrow = number_of_taxa_cat, ncol = number_of_r_values, dimnames = list(as.character(list_of_taxa), list_of_r))
  matrix_res_topo <- matrix(nrow = number_of_taxa_cat, ncol = number_of_r_values, dimnames = list(as.character(list_of_taxa), list_of_r))
  

  for (i in 1:length(list_of_r)){
    
    r_full_dir <-  paste(c(simulation_dir, list_of_r[i]), collapse ="/")

    for(j in 1:length(list_of_taxa)){
      average_dist <- 0
      average_dist_topo <- 0
     
      size_full_dir <-  paste(c(r_full_dir, list_of_taxa[j]), collapse ="/")
      
      rep_list <- list.dirs(path = size_full_dir, full.names = FALSE, recursive = FALSE)
      for (k in 1:length(rep_list)){
        rep_full_dir <-  paste(c(size_full_dir, rep_list[k]), collapse ="/")

        if (inner_dir){
          rep_full_dir <- paste(c(rep_full_dir, sub_dir), collapse = "/")
        }

        tree_path <- paste(c(rep_full_dir, find_tree_name(method, phyML)), collapse = "/")
        
        tree <- read.tree(tree_path)

        original_tree_path <- paste(c(simulation_dir, "1.0", list_of_taxa[j], rep_list[k], "tree.newick"), collapse = "/")
        original_tree <- read.tree(original_tree_path)

        distance <- KF.dist(tree, original_tree, rooted = TRUE)
        distance_topo <- RF.dist(tree, original_tree, rooted = TRUE)
          
        average_dist <- average_dist + distance
        average_dist_topo <- average_dist_topo + distance_topo


      }
      average_dist <- average_dist/number_of_replicates
      average_dist_topo <- average_dist_topo/number_of_replicates
      matrix_res[j,i] <- average_dist
      matrix_res_topo[j,i] <- average_dist_topo


    }
    
  }

  data_frame <- data.frame(matrix_res)
  data_frame_topo <- data.frame(matrix_res_topo)
  colnames(data_frame) <- list_of_r
  colnames(data_frame_topo) <- list_of_r



  if (!dir.exists(res_dir)){
    dir.create(res_dir)
  }

  method_dir <- paste(c(res_dir, sub_dir), collapse = "/")

  
  if (!dir.exists(method_dir)){
    dir.create(method_dir)
  }
  if (phyML){
    write.csv(data_frame, file = paste(c(method_dir, "phyML.csv"), collapse = "/"))
    write.csv(data_frame_topo, file = paste(c(method_dir, "phyML_topo.csv"), collapse = "/"))
  }else{
    write.csv(data_frame, file = paste(c(method_dir, "scaled.csv"), collapse = "/"))
    write.csv(data_frame_topo, file = paste(c(method_dir, "scaled_topo.csv"), collapse = "/"))
  }

}
######################################################################################################################

find_tree_name <- function(method, phyml){
  path <- NULL
  if (method == "treepl" || method == "treePL"){
    path <- "dated_treepl.newick"
  }else if (method == "r8s"){
    path <- "r8s_dated_tree.newick"
    
  }else if (method == "mrbayes"){
    path <- "parsemb_map_tree.tre"
  
  }else{
    path <- paste(c("dated_tree_",method, ".newick"), collapse = "")
    
  }
  if( phyml == 1){
    path <- paste(c("phyML", path), collapse = "_")
  }
  return (path)
}




#######################################################################################################################

args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
}



calculate_tree_distance(output_dir, res_dir, list_of_r, list_of_taxa, number_of_replicates, method, sub_dir, inner_dir, phyML)