library(diversitree)
library(phytools)
source("rescale_depth.R")

# Create a constraint file for phyML
create_constraint_file <- function(parent_dir_of_tree, tree_path, phyML){
  tree <- read.tree(tree_path)
  root_node <- get_root(tree)
  direct_descendants <- get_children(tree, root_node)
  descendants_of_first_node <- getDescendants(tree, node = direct_descendants[1])
  descendants_of_second_node <- getDescendants(tree, node = direct_descendants[2])
  number_of_taxa = length(tree$tip.label)
  first_monophyletic_group <- find_tips_among_descendants(tree, descendants_of_first_node, number_of_taxa)
  second_monophyletic_group <- find_tips_among_descendants(tree, descendants_of_second_node, number_of_taxa)
  
  # save the data structure for future use for rooting.
  saveRDS(first_monophyletic_group, paste(c(parent_dir_of_tree, "monophyletic_group1.rds"),collapse = "/"))
  saveRDS(second_monophyletic_group, paste(c(parent_dir_of_tree, "monophyletic_group2.rds"), collapse = "/"))
  # save data for treepl
  write_mrca_info(parent_dir_of_tree, first_monophyletic_group, second_monophyletic_group, phyML)
  #if(length(second_monophyletic_group) == 1){
    #write_constraint_file(path_of_constraint_file, second_monophyletic_group, first_monophyletic_group)
  #}else{
    #write_constraint_file(path_of_constraint_file, first_monophyletic_group, second_monophyletic_group)
  #}
  
  
}
##################################################################################################################
get_root <- function(phy){
  return(getMRCA(phy, phy$tip.label))
}

##################################################################################################################
write_mrca_info <- function(parent_dir_of_tree, first_monophyletic_group, second_monophyletic_group, phyML){
  name <- "mrca_treepl.txt"
  tip1 <- first_monophyletic_group[1]
  tip2 <- second_monophyletic_group[1]
  to_write <- c("rt", tip1, tip2)
  full_sentence <- paste(to_write, collapse = " ")
  if (phyML){
    name <- paste(c("phyML_", name), collapse = "")
  }
  full_name <- paste(c())
  output_file <- file(paste(c(parent_dir_of_tree, name), collapse = "/"))
  writeLines(c(full_sentence), output_file)
  close(output_file)
  
}
#################################################################################################################
# write newick file which will be the constraint file for phyML
write_constraint_file <- function(path_of_constraint_file, first_monophyletic_group, second_monophyletic_group){
  inner_group_as_string_vector <- c("(", paste(second_monophyletic_group, collapse = ","), ")")
  inner_group_as_string <- paste(inner_group_as_string_vector, collapse = "")
  outer_group_as_string_vector <- c("(", paste(first_monophyletic_group, collapse = ","))
  outer_groups_as_string <- paste(outer_group_as_string_vector, collapse = "")
  joined_str <- paste(c(outer_groups_as_string, inner_group_as_string), collapse = ",")
  joined_final_string <- paste(c(joined_str, ");\n"), collapse = "")
  output_file <- file(path_of_constraint_file)
  writeLines(c(joined_final_string), output_file)
  close(output_file)
}

#################################################################################################################

get_children <- function(phy, node){
  children <- c(0,0)
  edges <- phy$edge
  number_of_edges <- length(phy$edge.length)
  counter <- 0
  for (i in 1:number_of_edges){
    if(counter == 2){
      break
    }
    if(phy$edge[i,1] == node){
      counter = counter + 1
      children[counter] = phy$edge[i,2]
    }
    
  }
  return (children)
}


################################################################################################################

args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
}
create_constraint_file(parent_dir_of_tree, tree_path, phyML)
