import os
import argparse
from Bio import AlignIO
import re
from Bio.Alphabet import generic_dna



def get_dimensions(phylip_aln):
    file = open(phylip_aln, 'r')
    line = file.readline()
    file.close()
    pattern = re.compile("([\d]+)")
    dimensions = pattern.findall(line)
    return dimensions

def get_matrix(phylip_aln):
    matrix = ""
    file = open(phylip_aln, 'r')
    file.readline()
    p = re.compile("\*s1")
    white_spaces = re.compile("[\S]+")
    for line in file:
        if white_spaces.findall(line) == []:
            continue
        if "*s1" in line:
            line = re.sub(p, "ss1", line)
        matrix += line
    file.close()
    white_spaces_to_del = re.compile("[\s]+\n")
    matrix = re.sub(white_spaces_to_del, "\n", matrix)
    return matrix



def convert_alignment_format(output_dir, model):
    phylip_aln = os.path.join(output_dir, "output_TRUE_1.phy")
    dimensions = get_dimensions(phylip_aln)
    matrix_data = get_matrix(phylip_aln)
    if model == "correlated":
        mrbayes_dir = os.path.join(output_dir, "mrbayes_"+model)
    elif model == "strict":
        mrbayes_dir = os.path.join(output_dir, "mrbayes_"+model)
    else:
        mrbayes_dir = os.path.join(output_dir, "mrbayes_"+ model)
        #mrbayes_dir = os.path.join(output_dir, "mrbayes_ngen")

    if not os.path.exists(mrbayes_dir):
        os.makedirs(mrbayes_dir)
    nex_aln = os.path.join(output_dir, mrbayes_dir, "msa.nexus")
    file = open(nex_aln, 'w')
    content = ""
    content += "#NEXUS\nbegin data;\n\tdimensions ntax=" + dimensions[0] + " nchar=" + dimensions[1]+";\n\t"
    content += "format datatype=dna missing=? gap=-;\n"
    content += "matrix\n"
    content += matrix_data
    content += ";\nend;\n"
    file.write(content)
    file.close()

    #nex_aln = os.path.join(output_dir, mrbayes_dir, "msa.nexus")
    #AlignIO.convert(phylip_aln, "phylip-sequential", nex_aln, "nexus")
    return

def create_nexus_for_mrbayes(output_dir, tree_sizes, replicates, r_lst, model):

    for r in r_lst:
        output_directory = os.path.join(output_dir, str(r))

        for tree_size in tree_sizes:
            tree_dir = os.path.join(output_directory, str(tree_size))

            for replicate in range(1, replicates+1):
                tree_rep_dir = os.path.join(tree_dir, str(replicate))
                convert_alignment_format(tree_rep_dir, model)
    return




# The outputed trees of Mr bayes have square braces -> need to remove them!
def delete_square_braces(file_name):
    file = open(file_name, 'r')
    content = file.read()
    file.close()
    p = re.compile("(\[.*?\])")
    content = re.sub(p, "", content)
    file = open(file_name, 'r')
    file.write(content)
    file.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates MrBayes input")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--model", "-m", default = "IGR", help = "specify model")



    args = parser.parse_args()
    output = args.output
    tree_sizes = args.tree_sizes
    replicates = args.number_of_replicates
    r = args.rates_ratio
    model = args.model


    # output_dir, tree_sizes, replicates, rates, r, tree_length

    create_nexus_for_mrbayes(output, tree_sizes, replicates, r, model)
