library(BAMMtools)
library(coda)

## get priors estimators file, and then add the block to the template file (python file)

create_constrol_file <- function(rep_dir, tree_file){
  tree <- read.tree(tree_file)
  if (!check_tree(tree)){
      tree <- correct_ultrametricized_trees(tree)
      write.tree(tree, treefile)
  }


  setBAMMpriors(read.tree(tree_file), outfile = paste(c(rep_dir, "priors_set.txt"), collapse = "/")) ## setting priors

 
}
############################################################################################################################

###########################################################################################################################
## get MAP configuration of shifts ###
get_MAP_results <- function(rep_dir, treefile, expected_number_of_shifts){
  phy <- read.tree(treefile)
  eventdata <- paste(c(rep_dir, "event_data.txt"), collapse = "/")
  mcmcout <- read.csv(paste(c(rep_dir,"mcmc_out.txt"), collapse = "/"), header=T)
  burnstart <- floor(0.1 * nrow(mcmcout))
  postburn <- mcmcout[burnstart:nrow(mcmcout), ]
  effective_size_N_shifts <- effectiveSize(postburn$N_shifts)
  effective_size_logLik <- effectiveSize(postburn$logLik)
  if (effective_size_N_shifts < 200 | effective_size_logLik < 200){# should be at least 200 if the analysis was run long enough
    print("***** did not converge! *******")
    return(0)
  }
  
  ## getting MAP result ##
  edata <- getEventData(phy, eventdata, burnin = 0.1)
  best <- getBestShiftConfiguration(edata, expectedNumberOfShifts=expected_number_of_shifts)
  results_file <- paste(c(rep_dir, "BAMM_shifts_results.rds"), collapse = "/")
  saveRDS(best, results_file)
  return(1)

}

#############################################################################################################################
## check tree validity ####
check_tree <- function(tree){

  time_calibrated <- is.ultrametric(tree)
  no_polytomies <- is.binary(tree)
  zero_branch_lengths <- length(tree$edge.length[tree$edge.length == 0])
  if (!time_calibrated){
    return (FALSE)
  }else if (!no_polytomies){
    return (FALSE)
  }else if(zero_branch_lengths){
    return (FALSE)
  }
  return (TRUE)
}

###############################################################################################################################
correct_ultrametricized_trees <- function(tree){
  
  ## BAMM cannot receive inputs with zero-branch lengths. This is a possible correction
  for(e in 1:length(tree$edge.length)){
    if (tree$edge.length[e] == 0){
      tree$edge.length[e] = 10^-8
      if (e == 1){
        print("The tree has zero-branch lengths->>> correcting...\n")
      }
    }
  }
  depths_of_nodes <- node.depth.edgelength(tree)
  number_of_tips <- length(tree$tip.label)
  depths_of_tips <- depths_of_nodes[1:number_of_tips]
  
  max_depth <- max(depths_of_tips)
  edges <- tree$edge
  ## the tree should be percisely ultrametric
  for (i in 1:length(tree$edge.length)){
    if (tree$edge[i, 2] <= number_of_tips){
      tip <- tree$edge[i, 2]
      tree$edge.length[i] <- tree$edge.length[i] + (max_depth - depths_of_tips[tip])
    }
  }
  return (tree)
}
###############################################################################################################################
##check tree and get results ##
get_results <- function(rep_dir, treefile, expected_number_of_shifts){
  tree <- read.tree(treefile)
  
  valid <- check_tree(tree)
  if (!valid){
    return(0)
  }
  get_MAP_results(rep_dir, treefile, expected_number_of_shifts)
  return(1)
}
###############################################################################################################################
## compute Bayes Factors (are robust to different priors)
get_bayes_factors <- function (rep_dir, expected_number_of_shifts){
  mcmcout <- read.csv(paste(c(rep_dir,"mcmc_out.txt"), collapse = "/"), header=T)
  all_bayes_factors <- computeBayesFactors(mcmcout, expectedNumberOfShifts=expected_number_of_shifts, burnin=0.1)
  BF_relative_to_null <- all_bayes_factors[,1]
  best_model <- names(BF_relative_to_null[BF_relative_to_null == max(BF_relative_to_null)])
  saveRDS(as.integer(best_model), file = paste(c(rep_dir, "BF_best.rds"), collapse = "/"))
}


###############################################################################################################################
args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
}
if (create_control){
  create_constrol_file(rep_dir, treefile)

}else{
  get_results(rep_dir, treefile, expected_number_of_shifts)
  get_bayes_factors(rep_dir, expected_number_of_shifts)
  
}