import os
import re
import argparse
from subprocess import Popen


#####################################
### Control file for BAMM ###########
#####################################


## modeltype=speciationextinction
## treefile=<path of input tree>
## runInfoFilename=<output_file_path>
## sampleFromPriorOnly=0
## runMCMC=1
## loadEventData = use the default
## initializeModel=1
## seed <default>
## validateEventConfiguration <default>
## expectedNumberOfShifts = 1.0 for trees with <500 tips


def create_control_BAMM(directory_of_replicate, tree_file_path, template_path, priors_txt, number_of_generations, sampling_frequency, method, phyml, expected_number_of_shifts):
    mcmc_parameters = ["numberOfGenerations", "mcmcWriteFreq", "eventDataWriteFreq", "printFreq"]


    file = open(template_path, 'r')
    content_template = file.read()
    file_priors = open(priors_txt, 'r')
    content_priors = file_priors.read()
    file_priors.close()
    file.close()
    path_for_control_modified = os.path.join(directory_of_replicate, "control_file_BAMM"+ "_"+method+"_"+str(phyml)+ ".txt")
    content_to_control_file = re.sub("%%%%", tree_file_path, content_template)
    pattern_prior_block = re.compile("#[\s]+PRIORS.*?# Prior \(rate parameter of exponential\) on extinction rates", re.DOTALL)
    content_to_control_file = re.sub(pattern_prior_block, content_priors, content_to_control_file)
    content_to_control_file = re.sub("expectedNumberOfShifts[\s]+=[\s]+[\d]+\.*(\d)*", "expectedNumberOfShifts = "+ expected_number_of_shifts, content_to_control_file)
    content_to_control_file = re.sub("runInfoFilename[\s]+=[\s]+run_info.txt", "runInfoFilename = "+ \
                                     os.path.join(directory_of_replicate, "BAMM_run_info_"+ method+ "_"+ str(phyml)+".txt"), content_to_control_file)

    for i in range(len(mcmc_parameters)):
        p = re.compile(mcmc_parameters[i] +"[\s]+=[\s]+([\d]+)")
        if i == 0:
            replacement = number_of_generations
        else:
            replacement = sampling_frequency
        content_to_control_file = re.sub(p, mcmc_parameters[i]+" = "+replacement, content_to_control_file)

    file_control_modified =  open(path_for_control_modified, 'w')
    file_control_modified.write(content_to_control_file)
    file_control_modified.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="runs simulation")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--input_tree", "-i", help = "input tree file")
    parser.add_argument("--control_path", "-c", help = "control file path")
    parser.add_argument("--prior_file_path", "-pr", help = "prior file path")
    parser.add_argument("--generations", "-g", help = "number of generations in mcmc")
    parser.add_argument("--sampling_freq", "-s", help = "sampling frequency")
    parser.add_argument("--method", "-m", help = "time-calibration method")
    parser.add_argument("--phyml", "-p", help = "phyml")
    parser.add_argument("--expected_number_of_shifts", "-e", help = "expected number of shifts")


    args = parser.parse_args()
    directory_of_replicate = args.output
    tree_file_path = args.input_tree
    template_path = args.control_path
    priors_txt = args.prior_file_path
    number_of_generations = args.generations
    sampling_frequency = args.sampling_freq
    method = args.method
    phyml = args.phyml
    expected_number_of_shifts = args.expected_number_of_shifts


    create_control_BAMM(directory_of_replicate, tree_file_path, template_path, priors_txt, number_of_generations, sampling_frequency, method, phyml, expected_number_of_shifts)












