import re
import argparse
import os



def get_best_smooth_parameter(cv_file):
    failure = True
    pattern_smooth = re.compile("Optimum:[\s]+[\S]+[\s]+([\S]+)")
    file = open(cv_file, 'r')
    content = file.read()
    file.close()
    pattern_check = re.compile("log10[\s]+smooth[\s]+smooth[\s]+Sq Error[\s]+Chi Square Error[\s]+[-]+[\s]+([^\*]+)",
                         re.DOTALL)
    cv_values_content = pattern_check.findall(content)[0]
    cvs_per_smooth = cv_values_content.split('\n')[:-1]
    p = re.compile("[\S]+")
    lst_chi_sq = []
    lst_of_smooth = []
    failure_list = []
    for cv in cvs_per_smooth:
        smooth = float(p.findall(cv)[1])
        chi_sq = float(p.findall(cv)[3])
        status = p.findall(cv)[4]
        lst_chi_sq.append(chi_sq)
        lst_of_smooth.append(smooth)
        failure_list.append(status)
    if "(Good)" in failure_list:
        failure = False
    minimal_smoothing_param = lst_of_smooth[lst_chi_sq.index(min(lst_chi_sq))]
    optimum = float(pattern_smooth.findall(content)[0])
    if failure: # only if all is failure, the chosen smoothing parameter would be the minimal
        return minimal_smoothing_param, failure

    return optimum, failure

##################################################################

def get_tree(tree_file_path):
    tree_file = open(tree_file_path, 'r')
    tree_content = tree_file.read()
    tree_file.close()
    tree_pattern = re.compile("[\S]+")
    tree = tree_pattern.findall(tree_content)[0]
    return tree

################################################################

def get_number_of_sites(new_msa_path):
    #edit_alignment(alignment_file_path, new_msa_path)
    alignment_file = open(new_msa_path, 'r')
    alignment = alignment_file.read()
    alignment_file.close()
    pattern_num_sites = re.compile("[\d]+[\s]+([\d]+)")
    number_of_sites = pattern_num_sites.findall(alignment)[0]
    return number_of_sites

################################################################

def get_mrca(mrca_file_path):
    mrca_file = open(mrca_file_path,'r')
    mrca_content = mrca_file.read()
    mrca_file.close()
    pattern_mrca = re.compile("[\S]+[\s]+[\S]+[\s]+[\S]+")
    mrca = "mrca "+pattern_mrca.findall(mrca_content)[0]+";"
    return mrca

#################################################################



def write_control_cv(template_file_path, initial_control, mrca_file_path, tree_file_path, alignment_file_path):
    control_file = open(template_file_path, 'r')
    content_control = control_file.read()
    control_file.close()

    ### replace with current tree ###

    tree = get_tree(tree_file_path)
    tree_pattern_in_control = re.compile("\(.*?;")
    content_control = re.sub(tree_pattern_in_control, tree, content_control)

    ### replace with corrected number of sites ###

    number_of_sites = get_number_of_sites(alignment_file_path)
    sites_pattern_control = re.compile("nsites[\s]*=[\s]*[\d]+")
    content_control = re.sub(sites_pattern_control, "nsites = "+number_of_sites, content_control)

    ### replace rooting constraint ###

    mrca = get_mrca(mrca_file_path)
    pattern_mrca_control = re.compile("mrca[\s]+rt[\s]+[\S]+[\s]+[\S]+")
    content_control = re.sub(pattern_mrca_control, mrca, content_control)
    control_file = open(initial_control, 'w')
    control_file.write(content_control)
    control_file.close()


def write_conrol_for_run(final_control_file_path, mrca_file_path, tree_file_path, alignment_file_path, depth, cv_file, r8s_status):
    smooth_parameter, failure = get_best_smooth_parameter(cv_file)
    if not failure:
        status_str = "OK"
    else:
        status_str = "Failure"
    file = open(final_control_file_path, 'w')
    content = "#NEXUS\n\nbegin trees;\n"
    content += "tree rtree = "+ get_tree(tree_file_path) +"\nEnd;\n\n"
    content += 'begin r8s;\nblformat nsites='+get_number_of_sites(alignment_file_path) + ' lengths=persite;\n'
    content += 'collapse;\n'+ get_mrca(mrca_file_path)+"\n"
    content += 'fixage taxon=rt age='+ depth+';\n'
    content += 'set minRateFactor = 0.005;\n'
    content += 'set minDurFactor = 0.00000001;\n'
    content += 'set smoothing='+ str(smooth_parameter)+ ';\n'
    content += 'set checkgradient=yes;\nset ftol=1e-9;\ndivtime method=pl algorithm=tn;\nshowage;\n'
    content += 'describe plot=node_info;\ndescribe plot=chrono_description;\ndescribe plot=rato_description;\nend;\n'
    file.write(content)
    file_status = open(r8s_status, 'w')
    file_status.write(status_str)
    file_status.close()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates control file for r8s")
    parser.add_argument("--cv", "-v", type = int, help = "initial run with cv")
    parser.add_argument("--cv_file", "-i", help = "inital control file for cv")
    parser.add_argument("--final_control", "-f",  help = "final control")
    parser.add_argument('--mrca_file', '-m', help = 'mrca file')
    parser.add_argument('--alignment_file', '-a', help = 'alignment file')
    parser.add_argument('--tree_file', '-t', help = "tree file")
    parser.add_argument('--depth', '-d', help = "tree age")
    parser.add_argument('--template_file_cv', '-tp', help = "template_file_for_cv")
    parser.add_argument('--output_of_cv', '-o', help = "output_of_cv")
    parser.add_argument("--r8s_status", "-r", help = "directory of r8s")



    args = parser.parse_args()
    cv = args.cv
    initial_control_file = args.cv_file
    final_control = args.final_control
    mrca_file = args.mrca_file
    alignment_file = args.alignment_file
    tree_file = args.tree_file
    depth = args.depth
    template_file = args.template_file_cv
    output_of_cv = args.output_of_cv
    r8s_status = args.r8s_status

    if cv:
        write_control_cv(template_file, initial_control_file, mrca_file, tree_file, alignment_file)
    else:
        write_conrol_for_run(final_control, mrca_file, tree_file, alignment_file, depth, output_of_cv, r8s_status)














