# -*- coding: utf-8 -*-
"""
Created on Fri Jan 13 13:28:16 2017

@author: User
"""

import re
import argparse


def generate_control_file(tree_path, file_path, seqLength, num_of_rep, indel_rate, number_of_taxa = None):
    
    params_dic = create_dict()
    file = open(file_path, 'w')
    file.write("[TYPE] NUCLEOTIDE 1\n")
    file.write("\n[SETTINGS]\n")
    fill_settings_section(file)
    fill_model_section(params_dic, file, indel_rate)#, model)
    if not tree_path:
        fill_random_tree(file, number_of_taxa)
    else:
        fill_tree_section(tree_path, file)
    fill_partition_section(file, seqLength)
    fill_evolve_section(num_of_rep, file)
    file.close()
    return file_path


    
def fill_settings_section(file):
    file.write("\t[ancestralprint] NEW\n")
    file.write("\t[phylipextension] phy\n")
    file.write("\t[output]          PHYLIP\n")
    file.write("\t[fileperrep]      TRUE\n")
    file.write("\t[printrates] TRUE\n")
    file.write("\t[printcodonsasaminoacids] FALSE\n")
    
    
def fill_model_section(params_dic, file, inderate):#, model):
    # if model == "K80":
    #     file.write("\n[MODEL] K80\n\t")
    #     file.write("[submodel] K80 "+ write_params(["a", "b"], params_dic) + "\n")
    #
    # elif model == "HKY+G":
    #     file.write("\n[MODEL] HKY\n\t")
    #     file.write("[submodel] HKY " + write_params(["a", "b"], params_dic) +"\n\t")
    #     file.write("[rates] " + params_dic["rates"] + "\n\t")
    #     file.write("[statefreq] " + params_dic["statefreq"] + "\n")
    #
    # else:   #model is GTR+G
    #file.write("\n[MODEL] GTRindels\n\t")
    file.write("\n[MODEL] HKYindels\n\t")
    #file.write("[submodel] GTR " + write_params(["a", "b", "c", "d", "e", "f"], params_dic) + "\n\t")
    file.write("[submodel] HKY " + write_params(["a"], params_dic) + "\n\t")
    file.write("[indelmodel] POW 1.8 25\n\t")
    #file.write("[indelrate] 0.0005\n\t")
    file.write("[indelrate] "+inderate +"\n\t")
    file.write("[rates] " + params_dic["rates"] + "\n\t")
    file.write("[statefreq] " + params_dic["statefreq"] + "\n")

        

def write_params(lst_params, params_dic):
    lst_vals = []
    for param in lst_params:
        lst_vals.append(params_dic[param])
    return " ".join(lst_vals)

def fill_random_tree(file, number_of_taxa):
    file.write("[TREE] tree\n")
    file.write("\t[rooted] "+ str(number_of_taxa) +" 6.7 2.5 0.234 0.31\n")
    file.write("\t[treedepth] "+ str(10)+"\n")
    
    

def fill_tree_section(tree_file, file):
    file_tree = open(tree_file, 'r')
    tree_content = file_tree.read()
    file_tree.close()
    pattern = re.compile("\(.*?;", re.DOTALL)
    tree = pattern.findall(tree_content)[0]
    tree = adjust_tree(tree)
    
    
    new_file = open(tree_file, 'w')
    new_file.write(tree)
    new_file.close()
    file.write("[TREE] tree " + tree + "\n")



 
def fill_partition_section(file, seqLength):
    file.write("\n[PARTITIONS] job\n\t")

    file.write("[tree HKYindels " +seqLength + "]\n")

        
    
    
def fill_evolve_section(number_of_replicates, file):
    file.write("\n[EVOLVE] job " +  number_of_replicates +" output\n")
    
    
def reformat_num(number):
    num = float(number)
    reformatted_num = str("{0:.10f}".format(num))
    return reformatted_num

def reformat_branch(branch):
    fixed_branch = branch[0] + ":" + branch[1:]
    return fixed_branch

def adjust_tree(tree_str):
    tree_str = tree_str.replace('ROOT', '')
    internal_nodes_pattern = 'nd[0-9]+'
    pattern = re.compile(internal_nodes_pattern)
    tree_str = pattern.sub('', tree_str)
    reformatted_number = re.compile(r'\d+[\.\d+]*e\-\d+')
    for number in reformatted_number.findall(tree_str):
        reformatted_num = reformat_num(number)
        tree_str = tree_str.replace(number, reformatted_num, 1)
    comb_branch_pattern = re.compile('\)[0-9]+\.[0-9]+')
    for branch in re.findall(comb_branch_pattern, tree_str):
        reformatted_branch = reformat_branch(branch)
        tree_str = re.sub(re.escape(branch), reformatted_branch, tree_str)
    bootsrap_num_expr = re.compile(r"-{0,1}[0-9]+\.[0-9]+:")
    tree_str = bootsrap_num_expr.sub("", tree_str)
    return tree_str


# def adjust_tree(tree_str):
#     # if there is ROOT string in the end of the tree -> remove it
# 	tree_str = tree_str.replace('ROOT', '')
#     # remove internal node names
#     internal_nodes_pattern = 'N[0-9]+'
#     pattern = re.compile(internal_nodes_pattern)
#     tree_str = pattern.sub('', tree_str)
#
#
#     # reformat small numbers
# 	reformatted_number = re.compile(r'\d+[\.\d+]*e\-\d+')
#
#     for number in reformatted_number.findall(tree_str):
#         reformatted_num = reformat_num(number)
#         tree_str = tree_str.replace(number, reformatted_num, 1)
#
#     comb_branch_pattern = re.compile('\)[0-9]+\.[0-9]+')
#     for branch in re.findall(comb_branch_pattern, tree_str):
#         reformatted_branch = reformat_branch(branch)
#         tree_str = re.sub(re.escape(branch), reformatted_branch, tree_str)
#
#     #remove bootsrap numbers
# 	bootsrap_num_expr = re.compile(r"-{0,1}[0-9]+\.[0-9]+:")
#     tree_str = bootsrap_num_expr.sub("", tree_str)
#
#     return tree_str
def create_dict():
    dic = {}
    dic["a"] = "2"
    #dic["a"] = "6"
    #dic["b"] = "1.8"
    #dic["c"] = "1"
    #dic["d"] = "2"
    #dic["e"] = "1.4"
    #dic["f"] = "4"
    dic["rates"] = "0 0.5 4"
    dic["statefreq"] = "0.298 0.202 0.202 0.298" # T C A G
    return dic

# kappa = 2 Simulate HKY +G

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="counts some statistics")
    parser.add_argument("--tree_path", "-t", help = "tree path")
    parser.add_argument("--file_path", "-f",   help = "control file path")
    parser.add_argument("--rootlength", "-r",  help = "root length")
    parser.add_argument("--num_replicates", "-n", help = "number of replicates")
    parser.add_argument("--indel_rate", "-i", help = "indel rate")



    args = parser.parse_args()
    tree_path = args.tree_path
    file_path = args.file_path
    seqLength = args.rootlength
    num_of_rep = args.num_replicates
    indel_rate = args.indel_rate



    generate_control_file(tree_path, file_path, seqLength, num_of_rep, indel_rate)




