import re
import os
import argparse

## edit alignmet according to outgroup removal
def edit_alignment(alignment_file_with_outgroup, new_msa_path):
    pattern_of_outgroup = re.compile("\*s1.*?\n")
    file = open(alignment_file_with_outgroup, 'r')
    content = file.read()
    file.close()
    content = re.sub(pattern_of_outgroup, "", content)  #remove outgroup
    processed_msa = remove_unneeded_gaps(content)
    file = open(new_msa_path, 'w')
    file.write(processed_msa)
    file.close()

## remove gaps ##
def remove_unneeded_gaps(content):
    list_of_gap_indices = create_list_of_gap_indices(content)
    sp_seq = get_sp_seq_pairs(content)
    pattern_num_sp = re.compile("[\d]+[\s]+[\d]+")
    num_sp_sites = str(len(sp_seq))+"  " + str(len(sp_seq[0][1])-len(list_of_gap_indices))
    content = re.sub(pattern_num_sp, num_sp_sites, content)

    for i in range(len(sp_seq)):
        ori_seq = sp_seq[i][1]
        new_seq = ""
        for j in range(len(ori_seq)):
            if not j in list_of_gap_indices:
                new_seq += ori_seq[j]
        content = re.sub(re.compile(ori_seq), new_seq, content)
    return content


## get indices of gaps which present in all of the sequences ###
def create_list_of_gap_indices(alignment):
    pairs_sp_seq = get_sp_seq_pairs(alignment)
    sp1, seq1 = pairs_sp_seq[1]
    pairs_sp_seq = pairs_sp_seq[2:]
    indices = []
    for i in range(len(seq1)):
        if seq1[i] == '-':
            gapped_in_all = True
            for sp, seq in pairs_sp_seq:
                if seq[i] != '-':
                    gapped_in_all = False
                    break
            if gapped_in_all:
                indices.append(i)
    return indices

## get pairs of species and sequence in tuples ##
def get_sp_seq_pairs(content):
    pattern_sp_seq = re.compile("([\S]+)[\s]+([\S]+)")
    pairs_sp_seq = pattern_sp_seq.findall(content)[1:]
    return pairs_sp_seq


def remove_outgroup_from_alignment(output_dir, tree_sizes, replicates, r_lst):

    for r in r_lst:
        r_full_dir = os.path.join(output_dir, r)
        for taxa in tree_sizes:
            taxa_full_dir = os.path.join(r_full_dir, taxa)
            for i in range(1, replicates+1):
                rep_full_dir = os.path.join(taxa_full_dir, str(i))
                alignment_with_outgroup_path = os.path.join(rep_full_dir, "output_TRUE_1.phy")
                alignment_no_outgroup_path = os.path.join(rep_full_dir, "msa_no_outgroup.phy")
                edit_alignment(alignment_with_outgroup_path, alignment_no_outgroup_path)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates treePL control files")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--taxa", "-t", nargs = '+', help = "number of taxa")
    parser.add_argument("--number_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--r_list", "-r", nargs='+', help="list_of_r")

    args = parser.parse_args()
    output_dir = args.output
    taxa_lst = args.taxa
    replicates = args.number_replicates
    r_list = args.r_list

    remove_outgroup_from_alignment(output_dir, taxa_lst, replicates, r_list)


