import argparse
from subprocess import Popen
import os
from create_job_scripts import create_job_format_for_all_jobs

def get_MAP_tree(output_dir, tree_sizes, replicates, r_lst, replicate_to_start, sub_dir, dir_of_scripts):
    R_file = os.path.join(dir_of_scripts, "MAP_tree_lab.R")
    for r in r_lst:
        output_directory = os.path.join(output_dir, str(r))

        for tree_size in tree_sizes:
            tree_dir = os.path.join(output_directory, str(tree_size))

            for replicate in range(replicate_to_start, replicates+1):

                tree_rep_dir_dst = os.path.join(tree_dir, str(replicate), sub_dir)
                consensus_tree = os.path.join(tree_rep_dir_dst, 'msa.nexus.con.tre')
                if not os.path.exists(consensus_tree):
                    print(consensus_tree)
                    

                cmd  = ""
                cmd += "module load R/3.5.1\n"
                cmd += "cd " + tree_rep_dir_dst + "\n"
                path_map_tree = os.path.join(tree_rep_dir_dst, 'parsemb_map_tree.tre')

                if not os.path.exists(path_map_tree):
                    cmd += "R CMD BATCH '--args dir_path=\"" + tree_rep_dir_dst + "\" sum_of_logs=0"+ "'"+ " " + R_file + " " + os.path.join(tree_rep_dir_dst, "res_MAP_lab.out") + "\n"

                job_path = os.path.join(tree_rep_dir_dst, "edit_res_files.sh")
                job_file = open(job_path, "w")
                job_format_start = create_job_format_for_all_jobs(tree_rep_dir_dst, 'MAP')
                job_file.write(job_format_start)
                job_file.write(cmd)
                job_file.close()
                Popen(["qsub", "-p", "-1", job_path])

    return







if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates MrBayes input")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--rep_to_start", "-s", type=int, help="start run from replicate number x")
    parser.add_argument("--directory", "-d", help = "directory")
    parser.add_argument("--code_dir_of_scripts", "-c", help = "specify directory of scripts")



    args = parser.parse_args()
    output = args.output
    tree_sizes = args.tree_sizes
    replicates = args.number_of_replicates
    r = args.rates_ratio
    rep_to_start = args.rep_to_start
    directory = args.directory
    dir_of_scripts = args.code_dir_of_scripts


    # output_dir, tree_sizes, replicates, rates, r, tree_length

    get_MAP_tree(output, tree_sizes, replicates, r, rep_to_start, directory, dir_of_scripts)







