library(MEDUSA)
library(ape)


get_MEDUSA_inference <- function(tree_path, output_dir, method, phyml, correct_zero_branches = NULL){
  if(phyml == 1){
    phyml <- "phyML_"
  }else{
    phyml <- ""
  }
  if (!is.null(correct_zero_branches)){
    phyml <- paste(c("no_zero_branches", phyml), collapse = "_")
  }
  
  
  output_res_file_pars <- create_results_file_path(output_dir, phyml, method, "medusa_pars.rds")
  output_res_file_model <- create_results_file_path(output_dir, phyml, method, "medusa_model.rds")
  output_res_file_shifts <- create_results_file_path(output_dir, phyml, method, "medusa_shifts.rds")

  
  
  tree <- read.tree(tree_path)
  print(tree_path)
  tree <- correct_ultrametricized_trees(tree, correct_zero_branches)
  res <- MEDUSA(tree, model = "bd", resolveTree = TRUE)
  number_of_parameters <- res$optModel$num.par
  if (number_of_parameters > 2){
    saveRDS(1, output_res_file_model) # false positive
  }else{
    saveRDS(0, output_res_file_model) #true negative
  }
  saveRDS(res$optModel$par, output_res_file_pars)
  saveRDS(res$optModel$split.at, output_res_file_shifts)
  

  
}


############################################################################################################
create_results_file_path <- function(output_dir, phyml, method, file_name){
  short_file_name <- paste(c(phyml, method, "_", file_name), collapse = "")
  full_path_res <- paste(c(output_dir, short_file_name), collapse = "/")
  return (full_path_res)
  
}

##############################################################################################################


correct_ultrametricized_trees <- function(tree, correct_zero_branches = NULL){
  if (!is.null(correct_zero_branches)){
    for(e in 1:length(tree$edge.length)){
      if (tree$edge.length[e] == 0){
        tree$edge.length[e] = 10^-8
        if (e == 1){
          print("The tree has zero-branch lengths->>> correcting...\n")
        }
      }
    }
  }
  depths_of_nodes <- node.depth.edgelength(tree)
  number_of_tips <- length(tree$tip.label)
  depths_of_tips <- depths_of_nodes[1:number_of_tips]
  max_depth <- max(depths_of_tips)
  edges <- tree$edge
  for (i in 1:length(tree$edge.length)){
    if (tree$edge[i, 2] <= number_of_tips){
      tip <- tree$edge[i, 2]
      tree$edge.length[i] <- tree$edge.length[i] + (max_depth - depths_of_tips[tip])
    }
  }
  return (tree)
}

##############################################################################################################
correct_branch_lengths <- NULL
args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
}
# allow an option of branch length correction #
get_MEDUSA_inference(tree_path, output_dir, method, phyml, correct_branch_lengths)
