import os
import argparse
from subprocess import Popen
from create_job_scripts import create_job_format_for_all_jobs

### runs medusa analysis ###

def run_medusa_inference(output_directory, r_list, taxa_list, start, end, method, phyml, correct_branch_lengths, subdir, inner_dir, scripts_dir):
    R_script = os.path.join(scripts_dir, "medusa_inference.R")
    for r in r_list:
        r_directory = os.path.join(output_directory, str(r))
        for taxa in taxa_list:
            taxa_full_path = os.path.join(r_directory, str(taxa))
            for i in range(start, end+1):
                replicate_full_dir = os.path.join(taxa_full_path, str(i))
                if inner_dir:
                    replicate_full_dir = os.path.join(replicate_full_dir, subdir)
                    if method == "mrbayes":

                        MAP_dir = os.path.join(replicate_full_dir, "MAP_dir")
                        if not os.path.exists(MAP_dir):
                            os.makedirs(MAP_dir)

                tree_path = os.path.join(replicate_full_dir, find_tree_path_name(method, phyml))
                if phyml:
                    results_path = os.path.join(replicate_full_dir, "phyML_"+ method +"_medusa_model.rds")
                else:
                    results_path = os.path.join(replicate_full_dir, method +"_medusa_model.rds")
                if os.path.exists(results_path):
                    continue


                    #tree_path_sum_of_logs = os.path.join(replicate_full_dir, find_tree_path_name(method, phyml, 1))


                if not os.path.exists(tree_path):
                    print("doesn't exist : "+ tree_path+"\n")
                    continue

                #output_medusa_file = find_output_file_name(replicate_full_dir, phyml, method, int(correct_branch_lengths))#os.path.join(replicate_full_dir, phyML_treepl_medusa_model.rds)
                #if os.path.exists(output_medusa_file):
                    #continue
                job_path = os.path.join(replicate_full_dir, "medusa_inference_"+method+"_"+str(phyml)+"_ln.sh")
                job_header = create_job_format_for_all_jobs(job_path, "medusa_"+method+"_"+str(phyml)+ "_ln")
                job_file = open(job_path, "w")
                job_file.write(job_header)

                job_file.write("module load gcc/gcc-7.3.0 R/3.5.1\n")
                job_file.write("cd "+ replicate_full_dir +"\n")

                if method != "mrbayes":
                    if (int(correct_branch_lengths)):
                        job_file.write("R CMD BATCH '--args output_dir=" + '"' + replicate_full_dir + '"' + " tree_path=" + '"' + tree_path + \
                            '"' + " method=" + '"' + method + '"' + " phyml=" + str(phyml) + " correct_branch_lengths="+correct_branch_lengths+ "'" + " " + R_script + " " + os.path.join(replicate_full_dir, str(phyml) + "_" + method + "_" + "medusa_inference.out") + "\n")
                    else:

                        job_file.write("R CMD BATCH '--args output_dir=" + '"'+replicate_full_dir+ '"'+ " tree_path="+ '"' +tree_path+ \
                                '"' +" method="+ '"' +method+ '"'+" phyml="+ str(phyml)+"'"+" "+ R_script+ " " + os.path.join(replicate_full_dir, str(phyml)+ "_"+method+"_"+ "medusa_inference.out")+"\n")
                else:
                    if (int(correct_branch_lengths)):
                        job_file.write("R CMD BATCH '--args output_dir=" + '"' + MAP_dir + '"' + " tree_path=" + '"' + tree_path + \
                            '"' + " method=" + '"' + method + '"' + " phyml=" + str(phyml) + " correct_branch_lengths="+correct_branch_lengths+ "'" + " " + R_script + " " + os.path.join(MAP_dir, str(phyml) + "_" + method + "_" + "medusa_inference.out") + "\n")

                    else:

                        job_file.write("R CMD BATCH '--args output_dir=" + '"'+MAP_dir+ '"'+ " tree_path="+ '"' +tree_path+ \
                                '"' +" method="+ '"' +method+ '"'+" phyml="+ str(phyml)+"'"+" "+ R_script+ " " + os.path.join(MAP_dir, str(phyml)+ "_"+method+"_"+ "medusa_inference.out")+"\n")

                job_file.close()
                Popen(["qsub", "-p", "-1", job_path])

def find_output_file_name(replicate_full_dir, phyml, method, correct_branch_lengths):


    path = method +"_medusa_model.rds"
    if phyml == 1:
        path = "phyML_"+ path
    if correct_branch_lengths:
        path = "no_zero_branches_"+ path
    return os.path.join(replicate_full_dir, path)



def find_tree_path_name(method, phyml):
    tree_path = None

    if method == "treepl":
        tree_path = "dated_treepl.newick"
    elif method == "pathd8" or method == "PATHd8":
        tree_path = "dated_tree_"+ method+".newick"
    elif method == "r8s":
        tree_path = "r8s_dated_tree.newick"
    else:
        tree_path = "parsemb_map_tree.tre"
    if phyml:
        tree_path = "phyML_"+ tree_path

    return tree_path



if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates MrBayes input")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--rep_to_start", "-s", type=int, help="start run from replicate number x")
    parser.add_argument("--rep_to_stop", "-e", type=int, help="stop run at replicate number x")
    parser.add_argument("--method", "-m", help="ultrametrization method")
    parser.add_argument("--phyml", "-p", type=int, help="whether the trees are inferred by phyML")
    parser.add_argument("--zero_branch_correction", "-z", help = "specify whether to correct zero branch lengths")
    parser.add_argument("--subdir", "-d", help = "sub directory")
    parser.add_argument("--inner_dir", "-i", type = int,  help = "whether the results should be stored in inner directory")
    parser.add_argument("--dir_of_scripts", "-f",  help = "directory where the scripts are located")




    args = parser.parse_args()
    output_dir = args.output
    taxa_list = args.tree_sizes
    r_list = args.rates_ratio
    start = args.rep_to_start
    end = args.rep_to_stop
    method = args.method
    phyml = args.phyml
    correct_branch_lengths = args.zero_branch_correction
    subdir = args.subdir
    inner_dir = args.inner_dir
    scripts_dir = args.dir_of_scripts


    # output_dir, tree_sizes, replicates, rates, r, tree_length

    run_medusa_inference(output_dir, r_list, taxa_list, start, end, method, phyml, correct_branch_lengths, subdir, inner_dir, scripts_dir)
#"R CMD BATCH '--args output_dir=" + replicate_full_dir+ " tree_path="+tree_path+" method="+ method+" phyml="+ phyml+"'"+" "+ R_script+ " " + os.path.join(replicate_full_dir, str(phyml)+ "_"+method+"_"+ "medusa_inference.out")
