import os
import re
import argparse
from subprocess import Popen
from create_job_scripts import create_job_format_for_all_jobs



def run_simulations(output_dir, tree_sizes, replicates, rates, r_lst, tree_depth, number_of_sites, scripts_dir, focal_trait):

    indel_rates = create_dict_indelrates()
    if focal_trait:
        R_file_BiSSE_simulation = os.path.join(scripts_dir, "simulate_focal_trait.R")
    else:
        R_file_BiSSE_simulation = os.path.join(scripts_dir, "simulate_hidden_trait.R")
    indelible_control_file_script = os.path.join(scripts_dir, "create_indelible_control.py")
    constraints_R_file = os.path.join(scripts_dir,"constraints_for_phyML.R")
    remove_outgroup_script = os.path.join(scripts_dir,"remove_outgroup.R")
    python_rename_file = os.path.join(scripts_dir,"conversions.py")
    python_reorder_file = os.path.join(scripts_dir,"reorder_alignment.py")

    for r in r_lst:
        output_directory = os.path.join(output_dir, str(r))
        if not os.path.exists(output_directory):
            os.makedirs(output_directory)
        for tree_size in tree_sizes:
            tree_dir = os.path.join(output_directory, str(tree_size))
            if not os.path.exists(tree_dir):
                os.makedirs(tree_dir)
            for replicate in range(1, replicates+1):
                tree_rep_dir = os.path.join(tree_dir, str(replicate))
                if not os.path.exists(tree_rep_dir):
                    os.makedirs(tree_rep_dir)
                elif os.path.exists(os.path.join(tree_rep_dir, "phyML_tree.newick")):
                    continue


                tree_for_indelible = os.path.join(tree_rep_dir, "tree_for_phyML.newick")
                file_path = os.path.join(tree_rep_dir, "control.txt")
                msa_file_path = os.path.join(tree_rep_dir, "output_TRUE_1.phy")
                job_path = tree_rep_dir + "/simulation.sh"
                job_file = open(job_path, "w")
                job_format_start = create_job_format_for_all_jobs(tree_rep_dir, 'simulation')
                job_file.write(job_format_start)
                job_file.write("cd " + tree_rep_dir + "\n")
                job_file.write("module load gcc/gcc-7.3.0\n")
                job_file.write("module load INDELible/INDELible-1.03\n")
                job_file.write("module load python/anaconda_python-3.5\n")

                cmd = ""
                cmd += "R CMD BATCH '--args number_taxa="+ str(tree_size)+ " rates=c"+"("+",".join(rates) + ")" + " r=" + str(r)+\
                                       " tree_length="+ tree_depth+ " tree_rep_dir="+"\""+tree_rep_dir+"\""+"' "+R_file_BiSSE_simulation+ " "+ os.path.join(tree_rep_dir, "output.out")+"\n"
                cmd += "python "+indelible_control_file_script +" -t "+tree_for_indelible + " -f "+file_path + " -r "+ str(number_of_sites) + " -n 1 -i "+str(indel_rates[tree_size]) + "\n"
                cmd += "echo " + file_path + " | "+"indelible\n"
                # for ultrametrization for true tree#
                cmd += "R CMD BATCH '--args parent_dir_of_tree="+"\""+tree_rep_dir+"\"" + " tree_path=" +"\""+ os.path.join(tree_rep_dir, "tree.newick")+"\""+\
                                         " phyML=0"+"' "+constraints_R_file+ " "+ os.path.join(tree_rep_dir, "output.out")+"\n"
                cmd += "python "+python_reorder_file+" -o " + msa_file_path + "\n"
                cmd += "PhyML" + " -i " + msa_file_path + " --sequential -d nt -b 0 -m 012345 --run_id " + str(replicate) +  " -f m -c 4 -a e -v e -t e -o tlr --no_memory_check\n"
                cmd += "python "+python_rename_file+ " -o "+ tree_rep_dir+" -a 0\n"
                cmd += "R CMD BATCH '--args phyML_tree_path="+"\""+ os.path.join(tree_rep_dir, "phyML_tree.newick")+ "\"' "+remove_outgroup_script+ " "+ os.path.join(tree_rep_dir, "output_outgroup.out")+"\n"
                # constraints for phyML ultrametrization #
                cmd += "R CMD BATCH '--args parent_dir_of_tree=" + "\"" + tree_rep_dir + "\"" + " tree_path=" + "\"" + os.path.join(
                    tree_rep_dir, "phyML_tree.newick") + "\""  + " phyML=1" + "' " + constraints_R_file + " " + os.path.join(
                    tree_rep_dir, "output.out") + "\n"

                job_file.write(cmd)
                job_file.close()

                Popen(["qsub", "-p", "-1", job_path])






def create_dict_indelrates():
    dic = {}
    dic[50] = 0.001
    dic[100] = 0.0005
    dic[200] = 0.0005
    dic[400] = 0.0005
    return dic

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="runs simulation")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--pars", "-p", nargs = "+", help = "model parameters")
    parser.add_argument("--tree_depth", "-l", help = "tree length")
    parser.add_argument("--scripts_path", "-d", help = "directory of scripts")
    parser.add_argument("--sequence_length", "-s", type = int, default = 500, help = "root length")
    parser.add_argument("--focal_trait", "-f", type=int, default=1, help = "whether to simulate the focal observed trait")


    args = parser.parse_args()
    output = args.output
    tree_sizes = args.tree_sizes
    replicates = args.number_of_replicates
    r = args.rates_ratio
    pars = args.pars
    tree_depth = args.tree_depth
    sequence_length = args.sequence_length
    scripts_dir = args.scripts_path
    focal_trait = args.focal_trait

    # output_dir, tree_sizes, replicates, rates, r, tree_length

    run_simulations(output, tree_sizes, replicates, pars, r, tree_depth, sequence_length, scripts_dir, focal_trait)



