import os
import re
from create_job_scripts import create_job_format_for_all_jobs
import argparse
from subprocess import Popen
import shutil


def run_BAMM(output_dir, tree_sizes, replicates, r_lst, methods, number_of_generations, sampling_frequency,
             expected_number_of_shifts, subdir, begin, inner_dir, model, phyml_or_not, scripts_dir):
    template_path = os.path.join(scripts_dir, "template_diversification.txt")

    #phyml_or_not = [0,1]
    for r in r_lst:
        r_full_dir = os.path.join(output_dir, str(r))
        for taxa in tree_sizes:
            taxa_full_dir = os.path.join(r_full_dir, str(taxa))
            for i in range(begin, replicates+1):
                rep_full_dir = os.path.join(taxa_full_dir, str(i))
                run_id = get_run_id(methods, model, phyml_or_not)
                cmd  = ""
                cmd += "module load gcc/gcc-7.3.0 R/3.5.1\n"
                cmd += "module load python/python-anaconda3.5\n"
                cmd += "cd " + rep_full_dir + "\n"
                blocks = ""
                
                for method in methods:
                    for phyml in phyml_or_not:
                        if method == "mrbayes" and phyml:
                            continue

                        block = get_block_of_cmds(rep_full_dir, method, template_path, number_of_generations, sampling_frequency, phyml, expected_number_of_shifts, subdir, inner_dir, scripts_dir, model)
                        if block == "":
                            continue
                        blocks += block
                        cmd += block
                if blocks == "":
                    continue
                job_path = os.path.join(rep_full_dir, "BAMM_"+run_id+ ".sh")
                job_file = open(job_path, "w")
                job_format_start = create_job_format_for_all_jobs(rep_full_dir, 'BAMM_'+run_id, ncpu= 4)
                job_file.write(job_format_start)
                job_file.write(cmd)

                job_file.close()
                Popen(["qsub",  job_path])

#############################################################################################
def get_run_id(methods, model, phyML_or_not):
    run_id = ""
    dic = {'treePL': '0', "pathd8": '1', "mrbayes":'2', "r8s": '3'}
    for method in methods:
        run_id += dic[method]
    if model:
        run_id += str(model)
    if len(phyML_or_not) == 2:
        return run_id
    elif phyML_or_not[0] == 0:
        return run_id+"_0"
    else:
        return  run_id+"_1"


######### get tree file path ##############################################################
def get_tree_path(rep_dir, method, phyml, bamm_dir):


    if method == "treePL":
        short_tree_path = "dated_treepl.newick"
    elif method == "pathd8" or method == "PATHd8":
        short_tree_path = "dated_tree_"+method+".newick"
    elif method == "r8s":
        short_tree_path = "r8s_dated_tree.newick"
    else:   ##mrbayes
        short_tree_path = "parsemb_map_tree.tre"
    if phyml:
        short_tree_path = "phyML_"+short_tree_path


    full_path_of_src = os.path.join(rep_dir, short_tree_path)
    if not os.path.exists(full_path_of_src):
        return None
    #if inner_dir:
        #rep_dir = os.path.abspath(os.path.join(rep_dir, os.pardir))
    full_path_of_dst = os.path.join(bamm_dir, short_tree_path)
    #print("src:", full_path_of_src)
    #print("dst:", full_path_of_dst)
    shutil.copyfile(full_path_of_src, full_path_of_dst)

    return full_path_of_dst


## write b;ock of commands for each method ####################################################
def get_block_of_cmds(rep_full_dir, method, template_path, number_of_generations, sampling_frequency, phy, expected_number_of_shifts, subdir, inner_dir, scripts_dir, model = 0):
    cmd = ""

    R_file = os.path.join(scripts_dir, "create_BAMM_control.R")
    python_file = os.path.join(scripts_dir, "create_BAMM_control.py")
    if phy:
        dir_for_BAMM_run = os.path.join(rep_full_dir,
                                            "BAMM_" + method + "_" + "phyML_" + expected_number_of_shifts)
    else:
        if method == "mrbayes":
            if model != 0:### IGR is specified with model = 0, because it was the first used model long ago in this project
                dir_for_BAMM_run = os.path.join(rep_full_dir, "BAMM_ln_" + subdir + "_" + expected_number_of_shifts)

            else:
                dir_for_BAMM_run = os.path.join(rep_full_dir, "BAMM_ln_" + method + "_" + expected_number_of_shifts)


        else:

            dir_for_BAMM_run = os.path.join(rep_full_dir, "BAMM_"+method+"_"+ expected_number_of_shifts)

    if not os.path.exists(dir_for_BAMM_run):
        os.makedirs(dir_for_BAMM_run)
    else:
        if os.path.exists(os.path.join(dir_for_BAMM_run, 'BAMM_shifts_results.rds')):
            return cmd

    if inner_dir:
        rep_full_dir = os.path.join(rep_full_dir, subdir)

    tree_file_path = get_tree_path(rep_full_dir, method, phy, dir_for_BAMM_run)
    if not tree_file_path:
        return cmd


    cmd += ("cd "+ dir_for_BAMM_run+"\n")
    cmd += "R CMD BATCH '--args create_control=1 rep_dir=\""+ dir_for_BAMM_run+ "\" treefile=\""+ tree_file_path+"\""+" expected_number_of_shifts="+ expected_number_of_shifts+ "'"+ " "+ R_file + " "+ os.path.join(dir_for_BAMM_run, "output_file_BAMM.out")+"\n"
    cmd += "python "+ python_file+ " -o "+ dir_for_BAMM_run+ " -i "+ tree_file_path+" -c "+ template_path + " -pr " + os.path.join(dir_for_BAMM_run, "priors_set.txt")+ " -g "+ \
           number_of_generations+ " -s "+ sampling_frequency +" -m "+ method+" -p "+str(phy)+ " -e "+ expected_number_of_shifts+"\n"
    cmd += "bamm -c "+ os.path.join(dir_for_BAMM_run, "control_file_BAMM"+ "_"+method+"_"+str(phy) + ".txt")+"\n"
    cmd += "R CMD BATCH '--args create_control=0 rep_dir=\"" + dir_for_BAMM_run + "\" treefile=\"" + tree_file_path + "\""+\
           " expected_number_of_shifts="+expected_number_of_shifts+"'" + " " + R_file + " " + os.path.join(dir_for_BAMM_run, "output_file_BAMM_bf.out") + "\n"
    cmd += "cd ..\n"
    return cmd



if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="runs simulation")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--methods", "-m", nargs = "+", help = "methods")
    parser.add_argument("--generations", "-g", help = "number of generations")
    parser.add_argument("--sampling_freq", "-s", help = "sampling fraction in mcmc")
    parser.add_argument("--expected_number_of_shifts", "-e", help = "expected number of shifts")
    parser.add_argument("--subdir", "-d", help = "sub directory of mrbayes")
    parser.add_argument("--begin", "-b", type = int, help = "begin from rep")
    parser.add_argument("--inner_dir", "-i", type = int, help = "whether the results should be in the inner dir")
    parser.add_argument("--scripts_dir", "-f", help = "specify scripts directory")
    parser.add_argument("--clock", "-c", type = int, default = 0, help = "specify whether the model is other than igr")
    parser.add_argument("--phyML", "-p", type = int, default = 2, help = "phyML or not")




    args = parser.parse_args()
    output = args.output
    tree_sizes = args.tree_sizes
    replicates = args.number_of_replicates
    r = args.rates_ratio
    methods = args.methods
    number_of_generations = args.generations
    sampling_frequency = args.sampling_freq
    expected_number_of_shifts = args.expected_number_of_shifts
    subdir = args.subdir
    begin = args.begin
    inner_dir = args.inner_dir
    model = args.clock
    phyML = args.phyML
    scripts_dir = args.scripts_dir

    if phyML == 0:
        lst_of_phyML = [0]
    elif phyML == 1:
        lst_of_phyML = [1]
    else:
        lst_of_phyML = [0,1]




    # output_dir, tree_sizes, replicates, rates, r, tree_length

    run_BAMM(output, tree_sizes, replicates, r, methods, number_of_generations, sampling_frequency, expected_number_of_shifts, subdir, begin, inner_dir, model, lst_of_phyML, scripts_dir)









