import os
import argparse
import re
from subprocess import Popen
import numpy as np
#from create_job_scripts import create_job_format_for_all_jobs_mrbayes   ##(path, job_name)
from create_job_scripts import create_job_format_for_all_jobs



def create_mrbayes_txt(config_file, tree_size, model, depth = 1, sampling_frequency = 5000, number_of_generations = 2500000):

    file = open(config_file, 'w')
    cmd = set_evolutionary_model()
    if model == "IGR":
        cmd += set_IGR_clock_model()
    elif model == "strict":
        cmd += set_strict_clock_model()
    elif model == "correlated":
        cmd += set_correlated_clock_model()
    cmd += 'prset clockratepr=lognorm(-7.08069,2.458582)\n'
    cmd += 'constraint root=1-'+str(tree_size)+'\n'
    cmd += 'calibrate root=Fixed('+ str(depth) +')\n'
    cmd += 'calibrate ss1=Fixed('+str(1.2*int(depth))+')\n'
    cmd += 'prset topologypr=constraints(root)\n'
    cmd += 'prset nodeagepr=calibrated\n'
    cmd += 'delete ss1\n'
    cmd += ('mcmcp ngen='+str(number_of_generations) +' samplefreq='  +str(sampling_frequency) +' printfreq='+str(sampling_frequency)+'\n')
    cmd += 'mcmc\n'
    cmd += 'sump relburnin=yes burninfrac=0.25\n'
    cmd += 'sumt relburnin=yes burninfrac=0.25\n'
    cmd += 'quit\n'
    file.write(cmd)
    file.close()

def set_strict_clock_model():
    cmd = 'prset brlenspr=clock:uniform\n'
    cmd += 'prset clockvarpr=strict\n'
    return cmd        ### code for strict clock model- check if some of the specified parameters in IGR are also relevant here



def set_correlated_clock_model():
    cmd = 'prset brlenspr=clock:uniform\n'
    cmd += 'prset clockvarpr=tk02\n'
    return cmd

            ### code for correlated clock model- check if some of the specified parameters in IGR are also relevant here


def set_evolutionary_model():
    cmd = ""
    cmd += 'set autoclose=yes nowarn=yes\n'
    cmd += 'execute msa.nexus\n'
    cmd += 'lset nst=6 rates=invgamma\n'
    return cmd

def set_IGR_clock_model():
    cmd = 'prset brlenspr=clock:uniform\n'
    cmd += 'prset clockvarpr=igr\n'
    return cmd



def create_job_mrbayes(tree_rep_dir):
    job_path = os.path.join(tree_rep_dir, 'mrbayes.sh')
    headers = create_job_format_for_all_jobs_queue(tree_rep_dir, 'MrBayes')
    job_file = open(job_path, 'w')
    job_file.write(headers)
    job_file.write("cd " + tree_rep_dir + "\n")
    # job_file.write("module load R\n")
    job_file.write("module load gcc/gcc-7.3.0\n")
    job_file.write("module load mrbayers/mrbayers-3.2.7\n")
    job_file.write("mb < mrbayes.txt > log.txt\n")
    job_file.close()
    return job_path





def create_commands_for_mrbayes(output_dir, tree_sizes, replicates, r_lst, replicate_to_start, depth, model):


    for r in r_lst:
        output_directory = os.path.join(output_dir, str(r))

        for tree_size in tree_sizes:
            tree_dir = os.path.join(output_directory, str(tree_size))

            for replicate in range(replicate_to_start, replicates+1):
                tree_rep_dir = os.path.join(tree_dir, str(replicate), "mrbayes_" + model)
                if not os.path.exists(tree_rep_dir):
                    os.makedirs(tree_rep_dir)
                config_file = os.path.join(tree_rep_dir, 'mrbayes.txt')
                summary_file = os.path.join(tree_rep_dir, 'msa.nexus.con.tre')
                if os.path.exists(summary_file):
                    continue

                create_mrbayes_txt(config_file, tree_size, model, depth)
                job_file_path = create_job_mrbayes(tree_rep_dir)
                Popen(["qsub", job_file_path])


    return






if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates MrBayes input")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--rep_to_start", "-s", type=int, help="start run from replicate number x")
    parser.add_argument("--depth", "-d", help = "depth of tree")
    parser.add_argument("--model", "-m", help = "specify model")



    args = parser.parse_args()
    output = args.output
    tree_sizes = args.tree_sizes
    replicates = args.number_of_replicates
    r = args.rates_ratio
    rep_to_start = args.rep_to_start
    depth = args.depth
    model = args.model


    create_commands_for_mrbayes(output, tree_sizes, replicates, r, rep_to_start, depth, model)