import os
import re
import argparse
from subprocess import Popen
from create_job_scripts import create_job_format_for_all_jobs


def run_ultrametricize(output_dir, depth, phyml, sites, list_of_r, tree_sizes, number_of_replicates, dir_of_scripts):
    tree_name = ""

    if phyml:
        tree_name = "phyML_"

    for r in list_of_r:
        full_r_dir = os.path.join(output_dir, str(r))
        if not os.path.isdir(full_r_dir):
            continue
        for tree_size in tree_sizes:
            full_tree_size = os.path.join(full_r_dir, str(tree_size))
            for tree_replicate in range(1, number_of_replicates+1):
                full_tree_rep_dir = os.path.join(full_tree_size, str(tree_replicate))

                tree_path = os.path.join(full_tree_rep_dir, tree_name+"tree.newick")
                if not os.path.exists(tree_path):
                    continue

                elif os.path.exists(os.path.join(full_tree_rep_dir, tree_name +"dated_tree_pathd8.newick")):
                    continue

                job = create_ultrametricize_job(full_tree_rep_dir, depth, tree_name, phyml,
                                                sites, dir_of_scripts)
                Popen(["qsub", "-p", "-1", job])

    return

def create_ultrametricize_job(full_tree_rep_dir, depth, tree_name, phyml,
                              sites, dir_of_scripts):

    file_name_sh = "pathd8.sh"
    python_script = os.path.join(dir_of_scripts, "edit_newick_tree.py")
    python_cmd = "python "+ python_script + " -o "+ full_tree_rep_dir + " -s "+ sites + " -d "+ depth+ " -p "+ str(phyml)+"\n"
    python_cmd += "PATHd8 "+ os.path.join(full_tree_rep_dir, tree_name+"control_pathd8.txt") + " "+ os.path.join(full_tree_rep_dir, tree_name +"pathd8_output.txt") +"\n"

    job_file = open(os.path.join(full_tree_rep_dir, file_name_sh), "w")
    cmd = create_job_format_for_all_jobs(full_tree_rep_dir, tree_name + "PATHd8")

    cmd += "module load python/anaconda_python-3.4.0\n"
    cmd += python_cmd
    cmd += "cd "+ full_tree_rep_dir+"\n"
    job_file.write(cmd)
    job_file.close()
    return os.path.join(full_tree_rep_dir, file_name_sh)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description="creates treePL control files")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--depth", "-d", help = "depth of tree")
    parser.add_argument("--phyml", "-p", type = int, help = "phyml tree")
    parser.add_argument("--sites", "-s", help = "number of sites")
    parser.add_argument("--tree_sizes", "-t", type = int, nargs = '+', help = "list of tree sizes")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--rates_ratio", "-r", type = float, nargs = '+', help = "r")
    parser.add_argument("--code_scripts_dir", "-c", help = "specify the directory of scripts")


    args = parser.parse_args()
    output_dir = args.output
    depth = args.depth
    phyml = args.phyml
    sites = args.sites
    tree_size = args.tree_sizes
    r = args.rates_ratio
    number_of_replicates = args.number_of_replicates
    dir_of_scripts = args.code_scripts_dir



    run_ultrametricize(output_dir, depth, phyml, sites, r, tree_size, number_of_replicates, dir_of_scripts)






