import os
import argparse
from subprocess import Popen
from create_job_scripts import create_job_format_for_all_jobs


def run_fit_model(output_dir, method, list_of_r, tree_sizes, replicates, hisse, subdir, inner_dir,
                  phyML, dir_of_scripts):


    for r in list_of_r:
        full_r_dir = os.path.join(output_dir, r)
        if not os.path.isdir(full_r_dir):
            continue

        for tree_size in tree_sizes:
            full_tree_size = os.path.join(full_r_dir, tree_size)
            for tree_replicate in range(1, replicates+1):
                full_tree_rep_dir = os.path.join(full_tree_size, str(tree_replicate))
                phyML_tree_path = os.path.join(full_tree_rep_dir, "phyML_tree.newick")

                if not os.path.exists(phyML_tree_path):
                    continue
                dated_tree_path = get_tree_path(full_tree_rep_dir, subdir, method, inner_dir, 0)
                results_file = get_result_file(full_tree_rep_dir, subdir, inner_dir, hisse, method, phyML)
                if os.path.exists(results_file):
                    continue
                if method != "mrbayes":
                    phyML_dated_tree_path = get_tree_path(full_tree_rep_dir, subdir, method, inner_dir, 1)
                    if not os.path.exists(phyML_dated_tree_path) and phyML != 0:
                        continue
                if not os.path.exists(dated_tree_path) and phyML != 1:
                    continue


                job = create_job_file(full_tree_rep_dir, method, hisse, subdir, inner_dir, phyML, dir_of_scripts)
                if not job:
                    print("ERROR occured!")
                    return

                Popen(["qsub", "-p", "-1", job])

def get_result_file(full_tree_rep_dir, subdir, inner_dir, hisse, method, phyML):
    if hisse:
        short_dir = "HiSSE"
    else:
        short_dir = "LRT_test"
    if method == "mrbayes":
        short_dir += "_ln"
    if inner_dir and hisse:
        results_path = os.path.join(full_tree_rep_dir, subdir, short_dir)
        target_short_dir = method+"_model_selection.csv"
    elif hisse:
        results_path = os.path.join(full_tree_rep_dir, short_dir)
        target_short_dir = method+"_model_selection.csv"
    elif inner_dir and not hisse:
        results_path = os.path.join(full_tree_rep_dir, subdir, short_dir)
        target_short_dir = method + "_false_positives.rds"
    else:
        results_path = os.path.join(full_tree_rep_dir, short_dir)
        target_short_dir = method + "_false_positives.rds"
    if phyML == 1:
        target_short_dir = "phyML_"+ target_short_dir
    results_path = os.path.join(results_path, target_short_dir)
    return results_path

def get_tree_path(full_tree_rep_dir, subdir, method, inner_dir, phyML):

    if method == "treePL":
        short_tree_path = "dated_treepl.newick"
    elif method == "pathd8" or method == "PATHd8":
        short_tree_path = "dated_tree_pathd8.newick"
    elif method == "r8s":
        short_tree_path = "r8s_dated_tree.newick"
    else:   ##mrbayes
        short_tree_path = "parsemb_map_tree.tre"
    if phyML:
        short_tree_path = "phyML_"+ short_tree_path
    if inner_dir:
        return os.path.join(full_tree_rep_dir, subdir, short_tree_path)
    else:
        return  os.path.join(full_tree_rep_dir, short_tree_path)


def create_job_file(full_tree_rep_dir, method, hisse, sub_dir, inner_dir, phyML, dir_of_scripts):
    cmd = ""
    if inner_dir:
        path_of_job = os.path.join(full_tree_rep_dir, sub_dir, method +"_res_job_MAP"+ sub_dir+ str(hisse)+".sh")
    else:
        path_of_job = os.path.join(full_tree_rep_dir, method + "_res_job_MAP.sh")
    if hisse:
        R_script = os.path.join(dir_of_scripts, "fit_bisse_hisse.R")
    else:

        R_script = os.path.join(dir_of_scripts, "bisse_LRT_and_maxLik_estimates.R")
    if inner_dir:
        dir_of_tree = os.path.join(full_tree_rep_dir, sub_dir)
    else:
        dir_of_tree = full_tree_rep_dir
    if hisse:
        dir_of_run = os.path.join(dir_of_tree, "HiSSE")
    else:
        dir_of_run = os.path.join(dir_of_tree,"LRT_test")
    if method == "mrbayes":
        dir_of_run += "_ln"
    real_tree_run =  "R CMD BATCH '--args output_dir="+"\""+dir_of_tree+"\"" + " phyml=0" +\
                                          " states_path="+ "\""+os.path.join(full_tree_rep_dir, "states.rds")+"\""\
                       " method="+"\""+method+"\"" + "' "+R_script+ " "+ os.path.join(dir_of_tree, method+str(hisse)+"_output_res0.out")+"\n"


    phyml_tree_run =  "R CMD BATCH '--args output_dir="+"\""+dir_of_tree+"\"" + " phyml=1" +\
                                           " states_path="+ "\""+os.path.join(full_tree_rep_dir, "states.rds")+"\""\
                      " method="+"\""+method+"\""+"' "+R_script+ " "+ os.path.join(dir_of_tree, method+str(hisse)+ "_output_res.out")+"\n"

    if phyML != 1:
        cmd += real_tree_run
    if method != 'mrbayes' and phyML != 0:
        cmd += phyml_tree_run

    if cmd == "":
        print("ERROR!")
        return None


    header = create_job_format_for_all_jobs(dir_of_tree, 'fit_bisse_'+sub_dir+ str(hisse)+"_MAP")

    with open(path_of_job, "w") as handle:
        handle.write(header)


        handle.write("module load python/anaconda_python-3.5\n")
        handle.write("cd " + full_tree_rep_dir + "\n")
        handle.write(cmd)
    return path_of_job


def remove_old_files(dir_of_run):
    if os.path.exists(dir_of_run):
        lst_of_files = os.listdir(dir_of_run)
        for file in lst_of_files:
            os.remove(os.path.join(dir_of_run, file))

# output_dir, phyml, depth, rates, states_path, true_tree_path


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="runs simulation")
    parser.add_argument("--output", "-o", help = "output directory")
    parser.add_argument("--method", "-m", help = "method of ultrametrization")
    parser.add_argument("--r_list", "-r", nargs = "+",  help = "r list")
    parser.add_argument("--taxa_list", "-t", nargs = "+",  help = "taxa list")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--hisse", "-hb", type = int, help = "true if hisse")
    parser.add_argument("--subdir", "-s", help = "sub dir of mrbayes")
    parser.add_argument("--inner_dir", "-i", type = int, help = "whether the results for the examined method are stored in inner dir")
    parser.add_argument("--phyML", "-phy", type = int, default = 2, help = "phyML or not")
    parser.add_argument("--dir_of_scripts", "-d", help = "specify the directory_of_scripts")


    args = parser.parse_args()
    output = args.output
    method = args.method
    r_list = args.r_list
    taxa_list = args.taxa_list
    replicates = args.number_of_replicates
    hisse = args.hisse
    subdir = args.subdir
    inner_dir = args.inner_dir
    phyml = args.phyML
    dir_of_scripts = args.dir_of_scripts

    # output_dir, tree_sizes, replicates, rates, r, tree_length

    run_fit_model(output, method, r_list, taxa_list, replicates, hisse, subdir, inner_dir, phyml, dir_of_scripts)
