import os
import argparse
from subprocess import Popen
from create_job_scripts import create_job_format_for_all_jobs
from edit_alignment_without_outgroup import edit_alignment
import random
from create_control_for_r8s import get_number_of_sites

def run_r8s(output_dir, list_of_r, tree_sizes, number_of_replicates, phyML, start, root_age, scripts_dir):

    if phyML:
        tree_name = "phyML_"

    else:
        tree_name = ""

    python_file_cmd = os.path.join(scripts_dir, "create_control_for_r8s.py")
    output_dir_short = os.path.basename(os.path.normpath(output_dir))

    for r in list_of_r:
        full_r_dir = os.path.join(output_dir, r)
        if not os.path.isdir(full_r_dir):
            continue
        for tree_size in tree_sizes:
            full_tree_size = os.path.join(full_r_dir, tree_size)

            for tree_replicate in range(start, number_of_replicates+1):
                full_tree_rep_dir = os.path.join(full_tree_size, str(tree_replicate))
                r8s_dir = os.path.join(full_tree_rep_dir, 'r8s')
                if not os.path.exists(r8s_dir):
                    os.makedirs(r8s_dir)

                configure_file_path_intial = os.path.join(r8s_dir, tree_name+"initial_control.r8s")

                template_file = os.path.join(scripts_dir, 'template.r8s')
                configure_file_path_final = os.path.join(r8s_dir, tree_name+"final_control.r8s")

                alignment_file = os.path.join(full_tree_rep_dir, 'msa_no_outgroup.phy')
                alignment_file_with_outgroup = os.path.join(full_tree_rep_dir, 'output_TRUE_1.phy')
                mrca_file_path = os.path.join(full_tree_rep_dir, tree_name +"mrca_treepl.txt")
                tree_path = os.path.join(full_tree_rep_dir, tree_name+"tree.newick")
                output_of_r8s_for_initial_step = os.path.join(r8s_dir, tree_name+ 'initial_output.out')
                output_of_r8s_final = os.path.join(r8s_dir, tree_name +'final_output.out')
                dated_tree_path = os.path.join(r8s_dir, tree_name+'r8s_dated_tree.newick')
                poly_dated = os.path.join(r8s_dir, tree_name+'r8s_dated_tree_poly.newick')
                status_file = os.path.join(r8s_dir, tree_name+'status_r8s.txt')
                if os.path.exists(configure_file_path_intial):
                    continue
                if os.path.exists(dated_tree_path):
                    continue
                remove_old_files([configure_file_path_intial, configure_file_path_final, output_of_r8s_for_initial_step,
                                 output_of_r8s_final, dated_tree_path, status_file, poly_dated])

                if not os.path.exists(alignment_file):
                    edit_alignment(alignment_file_with_outgroup, alignment_file)

                job = create_treepl_job(full_tree_rep_dir, r8s_dir, configure_file_path_intial, configure_file_path_final,\
                                        template_file, mrca_file_path, tree_path, root_age,
                                            dated_tree_path, python_file_cmd, alignment_file, output_of_r8s_for_initial_step,\
                                        output_of_r8s_final, tree_name, status_file, output_dir_short)

                Popen(["qsub", job])

    return


def remove_old_files(lst_of_files_to_del):
    for file_to_del in lst_of_files_to_del:
        if os.path.exists(file_to_del):
            os.remove(file_to_del)

def create_treepl_job(full_tree_rep_dir, r8s_dir, configure_file_path_intial, configure_file_path_final, template_file, mrca_file_path, tree_path, root_age,\
                                            dated_tree_path, python_file_cmd, alignment_file, output_of_r8s_for_initial_step, output_of_r8s_final, tree_name, status_file, output_dir_short):


    job_path = full_tree_rep_dir + "/"+tree_name+"r8s.sh"

    #job_header = create_job_format_for_all_jobs_per_specific_node(full_tree_rep_dir, tree_name + "r8s", node)
    job_header = create_job_format_for_all_jobs(full_tree_rep_dir, tree_name + "r8s_" + output_dir_short)
    job_file = open(job_path, "w")
    job_file.write(job_header)

    job_file.write("module load r8s1.81\n")
    job_file.write("module load python/anaconda_python-3.5\n")
    job_file.write("cd "+ r8s_dir+"\n")

    job_file.write("python "+python_file_cmd+ " -v 1 -i "+ configure_file_path_intial+ " -f "+ configure_file_path_final+ " -m "+mrca_file_path+ \
                    " -a "+ alignment_file+ " -t "+ tree_path + " -d "+root_age +" -tp "+ template_file+" -o "+output_of_r8s_for_initial_step+  " -r "+ status_file+"\n")
    job_file.write('r8s -b -f '+ configure_file_path_intial + ' > '+ output_of_r8s_for_initial_step +'\n')

    job_file.write("python "+ python_file_cmd+ " -v 0 -i "+ configure_file_path_intial+ " -f "+ configure_file_path_final+ " -m "+mrca_file_path+ \
                    " -a "+ alignment_file+ " -t "+ tree_path + " -d "+root_age +" -tp "+ template_file+" -o "+output_of_r8s_for_initial_step+  " -r "+ status_file+ "\n")

    job_file.write('r8s -b -f '+ configure_file_path_final + ' > '+ output_of_r8s_final +'\n')
    python_file_cmd = "python "+ os.path.join(scripts_dir, "get_dated_tree_r8s.py")
    job_file.write(python_file_cmd +" -i "+ output_of_r8s_final+ " -o "+ dated_tree_path+"\n")

    job_file.close()
    return job_path





if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="creates treePL control files")
    parser.add_argument("--output", "-o", help = "simulation directory")
    parser.add_argument("--r_list", "-r", nargs = "+",  help = "r list")
    parser.add_argument("--taxa_list", "-t", nargs = "+",  help = "taxa list")
    parser.add_argument("--number_of_replicates", "-n", type = int, help = "number of replicates")
    parser.add_argument("--phyML", "-p", type = int, help = "whether ML or scaled")
    parser.add_argument("--depth", "-d", help = "root age")
    parser.add_argument("--start", "-s", type = int, help = "start in replicate s")
    parser.add_argument("--code_scripts_dir", "-c", help = "specify here the directory where the scripts are located")

    args = parser.parse_args()
    output_dir = args.output
    r_list  = args.r_list
    taxa_list = args.taxa_list
    number_of_replicates = args.number_of_replicates
    phyML = args.phyML
    root_age = args.depth
    start = args.start
    scripts_dir = args.code_scripts_dir




    run_r8s(output_dir, r_list, taxa_list, number_of_replicates, phyML, start, root_age, scripts_dir)





