library(diversitree)
library(phytools)
source("rescale_depth.R")

# 
# create_tree_with_exact_num_taxa <- function(num_taxa, rates){
#   #rates <- c(0.3,0.3,0.1,0.1,0.1,0.1)
#   #rates order: lambda0, lambda1, miu0, miu1, q01,q10
#   t = tree.bisse(rates, max.t=30, max.taxa = num_taxa, x0=0)
#   while(length(t$tip.state) != num_taxa){
#     t = tree.bisse(rates, max.t=30, max.taxa = num_taxa, x0=0)
#   }
#   return (t)
#   
# }

# simulate_tree <- function(number_taxa, rates){
#   #tr <- create_tree_with_exact_num_taxa(number_taxa, rates)
#   nex
#   states <- tr$tip.state
#   proportion <- sum(states)/length(states)
#   while(proportion <= 0.1 | proportion >= 0.9 ){
#     nex
#     states <- tr$tip.state
#     proportion <- sum(states)/length(states)
#   }
#   return (tr)
# }

##### The finction is not used ###########################################################
run_BiSSE <- function(rates, replication, tree_size, tree_rep_dir){
  tree = simulate_tree(tree_size, rates)
  print(paste("directory is ", tree_rep_dir))
  
  write.nexus(tree, file = paste(c(tree_rep_dir, "tree.nex"), collapse = "/"))
  p = starting.point.bisse(tree)
  neutral_states = tree$tip.state
  likelihood_free = make.bisse(tree, neutral_states) # likelihood function
  fit_likelihood_free = find.mle(likelihood_free, p)  # find mle parameters
  likelihood_constraint = constrain(likelihood_free, lambda0 ~ lambda1, mu0 ~ mu1)  # null model
  fit_liklihood_constraint = find.mle(likelihood_constraint, p[argnames(likelihood_constraint)]) # mle of null model
  AIC_results = AIC(fit_liklihood_constraint, fit_likelihood_free) # calculate AIC
  if (AIC_results$AIC[[2]] < AIC_results$AIC[[1]]){ # if the AIC of the alternative is lower than that of the null-> false positive
    saveRDS(1, paste(c(tree_rep_dir, "hypothesis.rds"),collapse = "/") )
    saveRDS(fit_likelihood_free, paste(c(tree_rep_dir,"mle.rds"), collapse = "/"))
  }else{
    saveRDS(0, paste(c(tree_rep_dir, "hypothesis.rds"),collapse = "/"))
    saveRDS(fit_liklihood_constraint, paste(c(tree_rep_dir,"mle.rds"), collapse = "/"))
  }
    
}

########################################################################################
## Given the index of the node, the function finds its name.


find_name_of_node <- function(node_index, phy){
  
  name <- NULL
  number_of_tips <- length(phy$tip.label)
  index_in_nodes <- node_index - number_of_tips
  if (index_in_nodes <= 0){
    name <- phy$tip.label[node_index]
  }else{
    name <- phy$node.label[index_in_nodes]
  }
  return (name)
  
}

############## 2 FUNCTIONS FOR TESTS ###############################################################

find_tree_with_more_than_2_states_change <- function(number_of_taxa){
  rates <- c(0.3, 0.3, 0.1, 0.1, 0.1, 0.1)
  tree <- tree.bisse(rates, max.t = Inf, max.taxa = number_of_taxa, x0=0)
  if (is.null(tree)){
    mapping <- NULL
  }else{
    mapping <- history.from.sim.discrete(tree, 0:1)
  }
  
  while(!check_if_more_than_2_changes(mapping, tree)){
    tree <- tree.bisse(rates, max.t = Inf, max.taxa = number_of_taxa, x0=0)
    counter = 0
    while(is.null(tree) & counter < 100){
      tree <- tree.bisse(rates, max.t = Inf, max.taxa = number_of_taxa, x0=0)
    }
    if (is.null(tree)){
      print("ERROR!")
      return (NULL)
    }
    mapping <- history.from.sim.discrete(tree, 0:1)
  }
  return(tree)
  
}

check_if_more_than_2_changes <- function(mapping, tree){
  if (is.null(mapping)){
    return (FALSE)
  }
  for( i in 1:length(tree$edge.length)){
    if (nrow(mapping$history[[i]]) > 2){
      return(TRUE)
      
      
    }
  }
  return(FALSE)
  
}


#################################################################################################

find_proportions <- function(p_matrix, mapping){
  if(length(mapping) == 1){
    if(names(mapping[1]) == "1"){
      p_matrix[1,1] <- 0
      p_matrix[2,1] <- 1
    }else{
      p_matrix[1,1] <- 1
      p_matrix[2,1] <- 0
    }
  }else{
    for (i in 1:length(mapping)){
      if(names(mapping[i]) == "0"){
        p_matrix[1,1] <- p_matrix[1,1] + (mapping[i]/ (sum(mapping)))
      }else{
        p_matrix[2,1] <- p_matrix[2,1] + (mapping[i]/ (sum(mapping)))
      }
      
      
      
    }

  }
  return (p_matrix)
}

#############################################################################################

# final rescaling according to tree length
rescale_according_to_length <- function(phy, sum_of_lengths, total_length){
  branch_lengths <- phy$edge.length
  for (i in 1:length(branch_lengths)){
    phy$edge.length[i] <- total_length*(phy$edge.length[i]/sum_of_lengths)
  }
  return(phy)
}

############################################################################################
get_tree_total_length <- function(phy){
  total <- 0
  branches <- phy$edge.length
  for ( i in 1:length(branches) ){
    total <- total + branches[i]
  }
  return(total)
}
###########################################################################################################

# rescale tree branch lengths according to r and B
change_branch_lengths <- function(phy, mapping, r, depth){
  
  tree <- scale_depth(phy, depth)## rescale the ultrametric tree to a specific depth ##
  tree_length <- get_tree_total_length(tree)## get length of tree ##
  
  #rescaled_tree <- change_branch_lengths(tree, mapping, r, tree_depth)
  
  edges <- phy$edge
  branch_lengths <- phy$edge.length
  sum_of_lengths = 0  # represents the tree length of the stretched tree
  
  for (i in 1:length(branch_lengths)){
    proportions_matrix <- matrix(c(0,0,0,0), nrow = 2, ncol = 2)
    proportions_matrix[2,2] = 1 # state 1, the [1,2] index of the matrix is state 0

    mapping_of_branch <- mapping[[6]][[i]]
    if (is.null(mapping_of_branch)){
      print("Error in mapping")
    }
    proportions_matrix <- find_proportions(proportions_matrix, mapping_of_branch)
    tree$edge.length[i] <- tree$edge.length[i] * (proportions_matrix[2,1]*r + proportions_matrix[1,1])
    sum_of_lengths <- sum_of_lengths + tree$edge.length[i]
    
  }
  #print(paste("total length is :# ", total_length))
  #rescale_according_to_length(phy, sum_of_lengths, total_length)
  rescaled_tree <- rescale_according_to_length(tree, sum_of_lengths, tree_length)
  #rescaled_tree <- scale_depth(phy, depth)
  return(rescaled_tree)
  
}

####### Temporary functions #########################################


check_if_exists_zero_length_edge <- function(phy){
  branches <- phy$edge
  zero_length <- vector(mode="integer", length= nrow(branches))
  branch_lengths <- phy$edge.length
  number_of_branches <- nrow(branches)
  for (i in 1:number_of_branches){
    if (phy$edge.length[i] == 0){
      zero_length[i] = 1
    }
  }
  return (zero_length)
}


####################################################################

get_children <- function(phy, node){
  children <- c(0,0)
  edges <- phy$edge
  number_of_edges <- length(phy$edge.length)
  counter <- 0
  for (i in 1:number_of_edges){
    if(counter == 2){
      break
    }
    if(phy$edge[i,1] == node){
      counter = counter + 1
      children[counter] = phy$edge[i,2]
    }
    
  }
  return (children)
}
####################################################################
simulate_tree <- function(rates, taxa){
  tree <- NULL
  counter <- 0
  while(is.null(tree) & counter < 100){ # sometimes the resulted tree is null. This loop should fix it.
    tree <- tree.bisse(rates, max.t= Inf, max.taxa = taxa, x0=0)
    counter <- counter + 1
  }
  if (is.null(tree)){
    print("ERROR in creation of the tree!!!")
  }
  return (tree)
}

####################################################################

main <- function(number_taxa, rates, r, tree_depth, dir_tree_rep){
  node_for_graft <- NULL

  if(as.integer(r) == 1){
    #print(paste("number of taxa is: ", number_taxa))
    #print(paste("rates are: ", rates))
    tree <- simulate_tree(rates, number_taxa)
    true_mapping <- history.from.sim.discrete(tree, 0:1)
    history <- true_mapping$history
    write.tree(tree, paste(c(dir_tree_rep, "original_tree.newick"),collapse = "/"))
    #saveRDS(mapping, paste(c(dir_tree_rep, "mapping.rds"),collapse = "/"))
    states <- tree$tip.state
    sates_for_mapping <- sim.character(tree, pars = c(rates[5], rates[6]), x0=0, model="mk2")
    saveRDS(sates_for_mapping, paste(c(dir_tree_rep, "states_simulated.rds"),collapse = "/"))
    saveRDS(history, paste(c(dir_tree_rep, "true_mapping.rds"),collapse = "/"))
    tree <- read.tree(paste(c(dir_tree_rep, "original_tree.newick"),collapse = "/"))
    mapping<- make.simmap(tree, sates_for_mapping, model="ER", nsim=1)
  }
  
  ## rescale the ultrametric tree to a specific depth ##
  #tree <- scale_depth(tree, depth)
  ## get length of tree ##
  #tree_length <- get_tree_total_length(tree)
  #rescaled_tree <- change_branch_lengths(tree, mapping, r, tree_depth)
  else{
    replicate_num <- basename(dir_tree_rep)
    taxa_num <- basename(dirname(dir_tree_rep))
    taxa_directory <- dirname(dir_tree_rep)
    r_directory <- dirname(taxa_directory)
    simulations_directory <- dirname(r_directory)
    tree <- read.tree(paste(c(simulations_directory,'1.0', taxa_num, replicate_num, "original_tree.newick"),collapse = "/"))
    mapping <- readRDS(paste(c(simulations_directory,'1.0', taxa_num, replicate_num, "mapping.rds"),collapse = "/"))
    states <- readRDS(paste(c(simulations_directory,'1.0', taxa_num, replicate_num, "states.rds"), collapse = "/"))
  }
  rescaled_tree <- change_branch_lengths(tree, mapping, r, tree_depth)
  write.tree(rescaled_tree, paste(c(dir_tree_rep, "tree.newick"),collapse = "/")) # rescaled true tree
  saveRDS(states, paste(c(dir_tree_rep, "states.rds"),collapse = "/"))
  saveRDS(mapping, paste(c(dir_tree_rep, "mapping.rds"),collapse = "/"))
  max_depth <- get_depth_of_tree(rescaled_tree)
  b_tree <- create_tree_with_outgroup(rescaled_tree, rates, max_depth)
  
  tree_for_phyML_usage_path <- paste(c(dir_tree_rep, "tree_for_phyML.newick"), collapse = "/")# contains outgroup (for phyML)
  write.tree(b_tree, tree_for_phyML_usage_path)
  
}
##############################################################################################################
create_tree_with_outgroup <- function(rescaled_tree, rates, tree_depth){
  
  subtree_for_rooting <- simulate_tree(rates, 3)
  
  # change species names in the dummy subtree
  tip_names <- c("*s1", "*s2", "*s3")
  descendants_option1 <- getDescendants(subtree_for_rooting, 4)
  descendants_option2 <- getDescendants(subtree_for_rooting, 5)
  tips_to_drop <- descendants_option2
  if (length(descendants_option1) > length(descendants_option2)){# non- root internal node is needed
    node_for_graft <- 5
  }else{
    node_for_graft <- 4
    tips_to_drop <- descendants_option1
  }
  
  subtree_for_rooting$tip.label <- tip_names
  attributes(subtree_for_rooting$tip.label)$names <- tip_names
  b_tree <- bind.tree(subtree_for_rooting, rescaled_tree, where = node_for_graft, interactive = FALSE)
  for(i in 1:length(tips_to_drop)){
    b_tree <- drop.tip(b_tree, tip_names[tips_to_drop[i]])  # leave only one outgroup species
  }
  root <- getMRCA(b_tree, b_tree$tip.label)
  indices <- get_edges_from_root(b_tree, root)
  b_tree$edge.length[indices[1]] <- tree_depth * 1.2 # branch to a leaf
  b_tree$edge.length[indices[2]] <- tree_depth * 0.2
  
  return (b_tree)
}
##############################################################################################################
get_edges_from_root <- function(phy, root){
  vector_of_indices <- c(0,0)
  for(i in 1:length(phy$edge.length)){
    if(phy$edge[i, 1] == root){
      if(phy$edge[i,2] <= length(phy$tip.label)){
        vector_of_indices[1] <- i # edge directed to a leaf
      }else{
        vector_of_indices[2] <- i # edge directed to internal node
      }
    }
  }
  return(vector_of_indices)
}

###############################################################################################################

args <- commandArgs(TRUE)
for( i in 1:length(args) ){
  eval(parse(text = args[[i]]))
  }
main(number_taxa, rates, r, tree_length, tree_rep_dir)
